use core::fmt;

use axum::{
    http::StatusCode,
    response::{IntoResponse, Response},
};
use thiserror::Error;

#[derive(Error, Debug)]
pub enum AppError {
    InvalidCsrf(#[from] axum_csrf::CsrfError),
    Other(#[from] anyhow::Error),
}

impl AppError {
    fn status_code(&self) -> StatusCode {
        match self {
            AppError::Other(_) => StatusCode::INTERNAL_SERVER_ERROR,
            AppError::InvalidCsrf(_) => StatusCode::BAD_REQUEST,
        }
    }

    fn message(&self) -> String {
        match self {
            AppError::Other(e) => format!("something went wrong: {}", e),
            AppError::InvalidCsrf(e) => format!("unable to verify csrf: {}", e),
        }
    }
}

impl fmt::Display for AppError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}", self.message())
    }
}

impl IntoResponse for AppError {
    fn into_response(self) -> Response {
        (self.status_code(), self.message()).into_response()
    }
}
